// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.conditions_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';

import 'messages_lib.dart' as v0;

void main() {
  test('Publish events while there are no handlers', () {
    var bus = new Omnibus();

    // Event with data
    bus.publishEvent(new v0.Data({'a': 'a'}));

    // Empty event
    bus.publishEvent(new v0.Data({}));
  });

  test('Various conditions filtering', () {
    var bus = new Omnibus();

    // Bind some handlers.
    bus.bindEventHandler(
        v0.Abc.id, {'a': 'a', 'b': 'b', 'c': 'c'}, dummyHandler);
    bus.bindEventHandler(
        v0.Abc.id, {'a': 'a', 'b': 'b', 'c': 'a'}, dummyHandler);
    bus.bindEventHandler(
        v0.Abc.id, {'a': 'b', 'b': 'b', 'c': 'a'}, dummyHandler);

    // Publish some events.
    bus.publishEvent(new v0.Abc(a: 'a', b: 'b', c: 'c'));
  });

  test('Condition-less handlers', () {
    var bus = new Omnibus();

    bus.bindEventHandler(v0.MessageForAll.id, {}, mForEveryoneHandler);
    bus.publishEvent(new v0.MessageForAll('This message is for everyone!'));
  });
}

Future<Null> dummyHandler(OmnibusEvent event) async {
  expect(event.data['a'], equals('a'));
  expect(event.data['b'], equals('b'));
  expect(event.data['c'], equals('c'));
}

Future<Null> mForEveryoneHandler(OmnibusEvent event) async {
  expect(event.data['message'], equals('This message is for everyone!'));
}
